Tested x86_64-pc-linux-gnu, applying to trunk.

-- 8< --

In this testcase, the use of __FUNCTION__ is within a function parameter
scope, the lambda's.  And P1787 changed __func__ to live in the parameter
scope.  But [basic.scope.pdecl] says that the point of declaration of
__func__ is immediately before {, so in the trailing return type it isn't in
scope yet, so this __FUNCTION__ should refer to foo().

Looking first for a block scope, then a function parameter scope, gives us
the right result.

        PR c++/118629

gcc/cp/ChangeLog:

        * name-lookup.cc (pushdecl_outermost_localscope): Look for an
        sk_block.

gcc/testsuite/ChangeLog:

        * g++.dg/cpp0x/lambda/lambda-__func__3.C: New test.
---
 gcc/cp/name-lookup.cc                                | 8 +++++---
 gcc/testsuite/g++.dg/cpp0x/lambda/lambda-__func__3.C | 6 ++++++
 2 files changed, 11 insertions(+), 3 deletions(-)
 create mode 100644 gcc/testsuite/g++.dg/cpp0x/lambda/lambda-__func__3.C

diff --git a/gcc/testsuite/g++.dg/cpp0x/lambda/lambda-__func__3.C 
b/gcc/testsuite/g++.dg/cpp0x/lambda/lambda-__func__3.C
new file mode 100644
index 00000000000..50ad6e55c1b
--- /dev/null
+++ b/gcc/testsuite/g++.dg/cpp0x/lambda/lambda-__func__3.C
@@ -0,0 +1,6 @@
+// PR c++/118629
+// { dg-do compile { target c++11 } }
+
+void foo() {
+  []() -> decltype(+__FUNCTION__) { return nullptr; };
+}

base-commit: 2f62e66e1444d858e937faa7147f6cd06296b33b
diff --git a/gcc/cp/name-lookup.cc b/gcc/cp/name-lookup.cc
index 7fadbccfe39..1cd982e12d4 100644
--- a/gcc/cp/name-lookup.cc
+++ b/gcc/cp/name-lookup.cc
@@ -5204,9 +5204,11 @@ pushdecl_outermost_localscope (tree x)
   cp_binding_level *b = NULL;
   auto_cond_timevar tv (TV_NAME_LOOKUP);
 
-  /* Find the scope just inside the function parms.  */
-  for (cp_binding_level *n = current_binding_level;
-       n->kind != sk_function_parms; n = b->level_chain)
+  /* Find the block scope just inside the function parms.  */
+  cp_binding_level *n = current_binding_level;
+  while (n && n->kind != sk_block)
+    n = n->level_chain;
+  for (; n && n->kind != sk_function_parms; n = b->level_chain)
     b = n;
 
   return b ? do_pushdecl_with_scope (x, b) : error_mark_node;
-- 
2.49.0

Reply via email to